// -*- C++ -*-
/*!
 * @file  FingerPosition.cpp
 * @brief ModuleDescription
 * @date $Date$
 *
 * $Id$
 */

#include "FingerPosition.h"

// Module specification
// <rtc-template block="module_spec">
static const char* fingerposition_spec[] =
  {
    "implementation_id", "FingerPosition",
    "type_name",         "FingerPosition",
    "description",       "ModuleDescription",
    "version",           "1.0.0",
    "vendor",            "TSB",
    "category",          "TactileService",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

FingerPosition::FingerPosition(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_object_positionIn("object_position", m_object_position),
    m_finger_positionOut("finger_position", m_finger_position),
    m_tactileTriggerPort("tactileTrigger"),
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("object_position", m_object_positionIn);
  
  // Set OutPort buffer
  registerOutPort("finger_position", m_finger_positionOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  m_tactileTriggerPort.registerConsumer("tactilePort", "TactileService", m_tactilePort);
  
  // Set CORBA Service Ports
  registerPort(m_tactileTriggerPort);
  
  // </rtc-template>

}

FingerPosition::~FingerPosition()
{
}


/*
RTC::ReturnCode_t FingerPosition::onInitialize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FingerPosition::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FingerPosition::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FingerPosition::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t FingerPosition::onActivated(RTC::UniqueId ec_id)
{
  m_finger_position.data.length(20);
  on_execute_return_code = RTC::RTC_OK;
  cui_start();
  return RTC::RTC_OK;
}



RTC::ReturnCode_t FingerPosition::onDeactivated(RTC::UniqueId ec_id)
{
  cui_stop();
  return RTC::RTC_OK;
}



RTC::ReturnCode_t FingerPosition::onExecute(RTC::UniqueId ec_id)
{
  return on_execute_return_code;
}

void FingerPosition::cui_run()
{
	long i,j,k,data_point_num;	
	double raw_data[MAX_DATA_NUM];

	float dummyf1 = 0.0, dummyf2 = 0.0, dummyf3 = 0.0, dummyf4 = 0.0, dummyf5 = 0.0, dummyf6 = 0.0;
	char c_fing[] = "tactile_data";
	char c_fingNoTxt[20];	
	int c_No;	
	int dummy = 0;	
	int vin,vout;
  int data = 0 ;
  printf("1:set ModelID\n");
  printf("2:get ModelID\n");
	printf("3:set data \n");
	printf("4:get data \n");
  READ_INT_FROM_STDIN(data);

  try {
	_CORBA_ULong bcd;
  switch(data){
  
   case 1:
		printf("enter value\n");
		READ_INT_FROM_STDIN(vout);
		m_tactilePort->setModelID(vout);
	  break;

   case 2:
		 vin = m_tactilePort->getModelID();
		 printf("%d\n",vin);
      break;

	 case 3:
			//f[^ԍ̖₢킹
			printf("Input model and measured data No.\n");
			READ_INT_FROM_STDIN(c_No);		
			//f[^荞----------------------------------------------------------
			FILE *fd;
			sprintf(c_fingNoTxt,"%s%d.txt",c_fing,c_No);
			printf("\n%s--\n",c_fingNoTxt);
			fd = fopen(c_fingNoTxt,"r"); 
			if (fd == NULL) {
				printf("can't open %s in FingerPosition::cui_run()\n", c_fingNoTxt);
				break;
			}
			k=0;
			while(1){
				if(k > MAX_DATA_NUM) break;
				fscanf(fd,"%d,%f,%f,%f,%f,%f,%f",&dummy,&dummyf1,&dummyf2,&dummyf3,&dummyf4,&dummyf5,&dummyf6);
				if(dummy == EOF )break;

				raw_data[k]   = dummyf1; 
				raw_data[k+1] = dummyf2; 
				raw_data[k+2] = dummyf3;
				raw_data[k+3] = dummyf4; 
				raw_data[k+4] = dummyf5; 
				raw_data[k+5] = dummyf6;
				printf("%d [%f %f %f] \n",dummy, raw_data[k], raw_data[k+1], raw_data[k+2]);	
				printf("%d [%f %f %f] \n",dummy, raw_data[k+3], raw_data[k+4], raw_data[k+5]);	
				k+=6;

			}
			fclose(fd);
			data_point_num = k;

			//f[^̑M
			m_finger_position.data.length(data_point_num);
			for(j=0;j<data_point_num;j++){
				m_finger_position.data[j]   = raw_data[j];
			}
			m_finger_positionOut.write(m_finger_position); 	

			m_tactilePort->setModelID(c_No);

		 break;

	 case 4:
			m_object_positionIn.read();
			bcd = m_object_position.data.length(); 
		  
			_CORBA_ULong j;
			printf("%d:  ",bcd);
			for(j=0; j<bcd; j=j+1){
				printf("%f ",m_object_position.data[j]);
			}
			printf("\n");
		 break;

   default:
	   break;
  
  }
  } catch(...) {
	  printf("Exception occured in FingerPosition::cui_run()\n");
	  on_execute_return_code = RTC::RTC_ERROR;
  }
}


/*
RTC::ReturnCode_t FingerPosition::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FingerPosition::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FingerPosition::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FingerPosition::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t FingerPosition::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void FingerPositionInit(RTC::Manager* manager)
  {
    RTC::Properties profile(fingerposition_spec);
    manager->registerFactory(profile,
                             RTC::Create<FingerPosition>,
                             RTC::Delete<FingerPosition>);
  }
  
};


