// -*- C++ -*-
/*!
 * @file  TactileRecognition.cpp
 * @brief ModuleDescription
 * @date $Date$
 *
 * $Id$
 */

#include "TactileRecognition.h"
#include "SharedData.h"

int model_No = -1;
RTC::ReturnCode_t on_execute_return_code;

// Module specification
// <rtc-template block="module_spec">
static const char* tactilerecognition_spec[] =
  {
    "implementation_id", "TactileRecognition",
    "type_name",         "TactileRecognition",
    "description",       "ModuleDescription",
    "version",           "1.0.0",
    "vendor",            "TSB",
    "category",          "TactileService",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

TactileRecognition::TactileRecognition(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
		m_ModelID(-1),
    m_finger_positionIn("finger_position", m_finger_position),
    m_object_positionOut("object_position", m_object_position),
    m_tactile_triggerPort("tactile_trigger"),
		m_tactilePort(&m_ModelID),
    // </rtc-template>
	dummy(0)
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("finger_position", m_finger_positionIn);
  
  // Set OutPort buffer
  registerOutPort("object_position", m_object_positionOut);
  
  // Set service provider to Ports
  m_tactile_triggerPort.registerProvider("tactilePort", "TactileService", m_tactilePort);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  registerPort(m_tactile_triggerPort);
  
  // </rtc-template>

}

TactileRecognition::~TactileRecognition()
{
}


/*
RTC::ReturnCode_t TactileRecognition::onInitialize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TactileRecognition::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TactileRecognition::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TactileRecognition::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

RTC::ReturnCode_t TactileRecognition::onActivated(RTC::UniqueId ec_id)
{
  m_ModelID = -1;
  model_No = -1;

  result_data.clear();
  model_point.clear();
  data_point.clear();
  data_from_rtc.clear();

  m_object_position.data.length(20);
  on_execute_return_code = RTC::RTC_OK;
  main_cui_thread.cui_start();

  return RTC::RTC_OK;
}

RTC::ReturnCode_t TactileRecognition::onDeactivated(RTC::UniqueId ec_id)
{
  main_cui_thread.cui_stop();

  return RTC::RTC_OK;
}

RTC::ReturnCode_t TactileRecognition::onExecute(RTC::UniqueId ec_id)
{
  if (on_execute_return_code != RTC::RTC_OK) return on_execute_return_code;
  
  long j,k;
  result_data.lock();
  for(j=0;j<20;j++){
		m_object_position.data[j] = result_data.result[j];
  }
  result_data.unlock();
  m_object_positionOut.write(m_object_position); 	
	
  m_finger_positionIn.read();
  _CORBA_ULong bcd = m_finger_position.data.length(); 
  m_finger_position.data.length(bcd);
  
  _CORBA_ULong n;

  if(m_ModelID == 99) {
    printf("%d: \n",bcd);
    
    for(n=0; n<bcd; n=n+6){
      printf("%f ",m_finger_position.data[n]);
      printf("%f ",m_finger_position.data[n+1]);   
      printf("%f ",m_finger_position.data[n+2]);
      printf("%f ",m_finger_position.data[n+3]);
      printf("%f ",m_finger_position.data[n+4]);   
      printf("%f\n",m_finger_position.data[n+5]);
    }	
  }
  
  data_from_rtc.lock();
  if(m_ModelID != -1){
    model_No = m_ModelID;
    data_from_rtc.num = (int)bcd;
    
    k = 0;
    for(n=0; n<bcd; n=n+6){
      data_from_rtc.d1[k]   = m_finger_position.data[n];
      data_from_rtc.d2[k]   = m_finger_position.data[n+1];
      data_from_rtc.d3[k]   = m_finger_position.data[n+2];
      data_from_rtc.d1[k+1] = m_finger_position.data[n+3];
      data_from_rtc.d2[k+1] = m_finger_position.data[n+4];
      data_from_rtc.d3[k+1] = m_finger_position.data[n+5];
      k = k + 2;
    }	
  }
  data_from_rtc.unlock();

  return on_execute_return_code;
}

/*
RTC::ReturnCode_t TactileRecognition::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TactileRecognition::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TactileRecognition::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TactileRecognition::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TactileRecognition::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void TactileRecognitionInit(RTC::Manager* manager)
  {
    RTC::Properties profile(tactilerecognition_spec);
    manager->registerFactory(profile,
                             RTC::Create<TactileRecognition>,
                             RTC::Delete<TactileRecognition>);
  }
  
};


